/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
*******************************************************************************
* Utility Library 
*******************************************************************************
* @file
* @ingroup util_base
* @brief Exception Class declaration 
*  
* - Supports: All
* - Created: 2005-08-06
* - Creator: Olivier Delannoy
* - $Revision $
* - Changelog: 
*    * 2005-08-10 : Adding OutOfBoundException class 
*    * 2005-08-06 : Moved to the Utility Library 
*    * 2005-05-29 : Initial version 
*******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

*******************************************************************************/
#ifndef UTIL_EXCEPTION_HH
#define UTIL_EXCEPTION_HH 1
#include "util_namespace.hh"
#include <exception>
#include <cstring> 
namespace Util{
    /**
     * @ingroup util_base
     * @brief base exception class for Util. 
     *
     *
     * All exception generated by the library relies on this
     * exception. All application should derived there exception from
     * this class. This class relies on the standard exception
     * mecanism. So all exceptions are catch and handled correclty
     * when used in an application based on the utility library. 
     *  
     */
    class UTIL_EXPORT Exception : public std::exception 
    {
    public:
        /**
         * Constructor 
         * @param message A string describing the exception 
         */
        explicit Exception(const String& message = "") throw();
        /**
         * Destructor
         */
        virtual ~Exception(void) throw();
        
        /** 
         * Return the message associated to the exception 
         */
        const char* what(void) const throw();
    protected:
        void append(const String& text);
    private:
        /** Store the exception message */
        String mMessage;
    }; 
    /**
     * @ingroup util_base
     * @brief Out of bound exception class 
     * 
     * This class is thrown each time an index is out of the range in
     * a container access.
     *
     */
    class UTIL_EXPORT OutOfBoundException : public Exception
    {
    public:
        /** 
         * Constructor 
         * @param message describing the exception 
         */
        explicit OutOfBoundException(const String& message = "") throw();
    protected:
    private:
    };  
    /** 
     * @ingroup util_base
     * @brief SystemException is used when a system call cause an error.
     *
     * This class is thrown each time a system call cause an error 
     */ 
    class UTIL_EXPORT SystemException : public Exception
    {
    public:
        /** 
         * Constructor 
         * @param err The content of errno 
         * @param message the associated message 
         */
        explicit SystemException(int32 err, const String& message) throw();
        /** 
         * return a readable description of the error that occurs 
         */
        const char* errorString(void) const throw();
        /** 
         * Get the error code associated to the system call 
         */
        int32 error(void) const throw();
    protected:        
    private:
        int32 mError;
    };
    /** 
     * @ingroup util_base
     * @brief This is a special case of SystemException cause by an IO operation.
     *
     * This exception class is used for every error related to IO operation. 
     */ 
    class UTIL_EXPORT IOException : public SystemException 
    {
    public:
        /**
         * Constructor
         * @param err the content of errno
         * @param message the associated message 
         */
        explicit IOException(int32 err, const String& message) throw();
    };    
    /** 
     * @ingroup util_base
     * @ingroup util_xml 
     * @brief Report error occuring while parsing an XML document.
     * 
     * This class is used to report any parsing error for XML file 
     * This can be cause due to malformed document or invalid document 
     */
    class UTIL_EXPORT XMLParseException : public Exception
    {
    public:
        /** 
         * Constructor 
         * @param row the row of the error 
         * @param column the column of the error 
         * @param message the explanation of the error 
         */
        explicit XMLParseException(uint32 row, uint32 column, const String& message)throw();
        /** 
         * Get the row of the error 
         */
        uint32 row(void) const throw();
        /**
         * Get the column of the error 
         */
        uint32 column(void) const throw();
    private:
        /** Store the row of the error */
        uint32 mRow;
        /** Store the column of the error */ 
        uint32 mColumn;
    };    
}
#ifdef UTIL_OPT_INLINE
#include "Exception.icc"
#endif
#endif
/*******************************************************************************/


    



