/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
*******************************************************************************
* Utility Library 
*******************************************************************************
* @ingroup util_template
* @file
* @brief Template InstanceCounter definition 
*  
* - Supports: ALL 
* - Created: 2005-08-06
* - Creator: Olivier Delannoy
* - $Revision $
* - Changelog: 
*    
*    * 2005-08-06 : Initial version  
*******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

*******************************************************************************/
#ifndef UTIL_COUNTER_HH
#define UTIL_COUNTER_HH 1
#include "util_namespace.hh"
namespace Util {
    /**
     * @ingroup util_template
     * @brief Instance Counter 
     * 
     * This is a template used to count objects instance.  This
     * template must be inherits using private inheritance. It's as
     * less intrusive as it can and take no more memory per instance.
     *
     * @note The inheritance modificator must but protected. 
     *
     * A trivial example is shown bellow 
     * @code
     * #include <iostream>
     * #include "InstanceCounter.hh"
     * class Test : protected Util::InstanceCounter<Test>
     * {
     * public:
     *     Test() {}
     *     ~Test() {}
     *     using Util::InstanceCounter<Test>::getCount;
     * protected:
     * private:
     * };
     *
     * using namespace std;
     *
     *
     * int main()
     * {
     *     Test a;
     *     Test b;
     *     Test c;
     *     Test *d = new Test;
     *     cout << Test::getInstanceCount() << endl;  // Must display 4
     *     delete d;
     *     cout << Test::getInstanceCount() << endl; // Must display 3
     *     return 0;
     * }
     * @endcode
     */
    template<class T> class InstanceCounter {
      
    public:
        /** 
         * Retrieve the number of instance 
         */
        static Util::uint32 getInstanceCount() {return mCount;}
    protected:
        /**
         * Default Constructeur 
         */
        InstanceCounter() {++mCount;}
        /** 
         * Copy Constructor
         */
        InstanceCounter(const InstanceCounter&) {++mCount;}
        /**
         * Destructor
         */
        virtual ~InstanceCounter() {--mCount;}
    private:
        /** 
         * Store the number of instance of the counted class 
         */
        static Util::uint32 mCount;
    };
      
    // Set the initial counter value to 0 
    template <class T> Util::uint32 InstanceCounter<T>::mCount = 0;
}
#endif 



