/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
*******************************************************************************
* Utility Library 
*******************************************************************************
* @ingroup util_system
* @file
* @brief Class Mutex declaration
*  
* - Supports:
*   - System 
*       - UNIX/Pthread (linux)
*       - Win32 (XP)
*   - SDL  (Linux)
* - Created: 2005-08-06
* - Creator: Olivier Delannoy
* - Changelog: 
*    
*    * 2005-08-06 : Initial version  
*******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

*******************************************************************************/
#ifndef UTIL_MUTEX_HH
#define UTIL_MUTEX_HH 1
#include "util_namespace.hh"
namespace Util {
    /** 
     * @ingroup util_system
     * @brief Critical Section Protection in Multithreaded application 
     * 
     * This class provides critical section protection using mutual
     * exclusion inside of a multi-threaded application.  This class
     * might rely on the system wide thread API or on SDL. The
     * end-user does not have to be changed depending on the
     * underlying thread api in use. 
     */ 
    class UTIL_EXPORT Mutex  
    {
    public:
        /**
         * Default Constructor
         */
        Mutex(void);
        /** 
         * Destructor 
         */
        ~Mutex(void);
        /** 
         * Lock the mutex. 
         * This is blocking call. 
         * Once this call returns you are the 
         * only thread to have the mutex. 
         */
        void lock(void);
        /** 
         * Unlock the mutex. 
         * Once finishing doing operation on shared data you 
         * an release the lock you get on a mutex. If you do not 
         * currently have the lock this function does nothing. 
         */ 
        void unlock(void);
    protected:
    private:
        /**
         * @ingroup util_system
         * @brief Mutex Implementation abstract class 
         * 
         * Mutex implementation abstract class hides the
         * implementation specific aspect of a Mutex.
         */ 
        class MutexImpl
        {
        public:
            /** 
             * Acquire a lock on the mutex 
             */
            virtual void lock(void) = 0;
            /**
             * Release a lock on the mutex 
             */
            virtual void unlock(void) = 0;
        };
        /** A pointer to the implementation object */
        MutexImpl* mImpl;
    };
}
#ifdef UTIL_OPT_INLINE
#  include "Mutex.icc"
#endif 
#endif 
/*******************************************************************************/      



