/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @ingroup util_tools
 * @brief Class Properties declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2005-08-09
 * - Creator: Olivier Delannoy
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2005-08-09 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_PROPERTIES_HH
#define UTIL_PROPERTIES_HH 1
#include "util_namespace.hh"
#include "Exception.hh"
#include <iostream>
#include <map> 

namespace Util 
{
    /**
     * @ingroup util_tools
     * @brief Provides an extended interface for the template specialization map<String, String> 
     *
     * 
     * - Supports: All
     * - Author: Olivier Delannoy
     * - Creator: 2005-08-09
     * - $Revision $
     */
    class UTIL_EXPORT Properties : public std::map<String, String>
    {
    public:
        /**
         * Default constructor 
         */
        Properties(void);
        /**  
         * Destructor 
         */
        virtual ~Properties(void);
        
        /** 
         * Copy constructor 
         * @param src a reference to the object to copy 
         */
        Properties(std::map<String, String>& src);
        /** 
         * Assignment operator 
         * @param src a reference to the object to assign from 
         */
        Properties& operator=(std::map<String, String>& src);
        /**
         * Test whether a key exists or not 
         */
        bool exists(const String& key) const throw();
        /** 
         * Add a new tupple key/value 
         * if key exists the last addition erase the previous one. 
         */
        void add(const String& key, const String& value);
        /**
         * Remove an existing key 
         */
        void remove(const String& key) throw();
        /**
         * Retrieve the key corresponding to index @em index
         * @param index the range of the key 
         * @return the key string corresponding to index. 
         * @throw OutOfBoundException if @em index outside of [0,count()[
         */
        const String& key(uint32 index) const throw(OutOfBoundException);
        /**
         * Retrieve the value corresponding to index @em index  as a string
         * @param index the range of the element expected 
         * @return the string value corresponding to @em index 
         * @throw OutOfBoundException if @em index outside of [0,count()[
         */
        const String& stringValue(uint32 index) const throw(OutOfBoundException);
        /**
         * Retrive the value corresponding to key @em key as a string 
         * @param key the name of the property 
         * @param defValue the default value if key does not exists 
         * @return the value corresponding to key if found or defValue
         */
        const String& stringValue(const String& key, const String& defValue="") const throw();
        /**
         * Retrieve the value corresponding to index @em index as an integer 
         * @param index the range of the element expected 
         * @return the string value corresponding to @em index 
         * @throw OutOfBoundException if @em index outside of [0,count()[
         */
        int32 intValue(uint32 index) const throw(OutOfBoundException);
        /**
         * Retrive the value corresponding to key @em key as an integer 
         * @param key the name of the property 
         * @param defValue the default value if key does not exists 
         * @return the value corresponding to key if found or defValue
         */
        int32 intValue(const String& key, int32 defValue=0) const throw();
        /**
         * Retrieve the value corresponding to index @em index as a boolean 
         * @param index the range of the element expected 
         * @return the string value corresponding to @em index 
         * @throw OutOfBoundException if @em index outside of [0,count()[
         */
        bool booleanValue(uint32 index) const throw(OutOfBoundException);
        /**
         * Retrive the value corresponding to key @em key as a boolean 
         * @param key the name of the property 
         * @param defValue the default value if key does not exists 
         * @return the value corresponding to key if found or defValue
         */
        bool booleanValue(const String& key, bool defValue=false) const throw();
        /**
         * Retrieve the value corresponding to index @em index as a
         * floatting point number
         *
         * @param index the range of the element expected 
         * @return the string value corresponding to @em index 
         * @throw OutOfBoundException if @em index outside of [0,count()[
         */
        real32 realValue(uint32 index) const throw(OutOfBoundException);
        /**
         * Retrive the value corresponding to key @em key as a 
         * floatting point number
         * @param key the name of the property 
         * @param defValue the default value if key does not exists 
         * @return the value corresponding to key if found or defValue
         */
        real32 realValue(const String& key, real32 defValue=0.0f) const throw();
        /**
         * Printout the object to a stream 
         * 
         * @param out the stream object 
         */
        virtual void print(std::ostream& out) const throw();
        
    protected:
    private:
    };
}
std::ostream& operator<<(std::ostream&, const Util::Properties& obj);
#ifdef UTIL_OPT_INLINE
#include "Properties.icc"
#endif
#endif // UTIL_PROPERTIES_HH
/*******************************************************************************/



