/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
*******************************************************************************
* Utility Library 
*******************************************************************************
* @ingroup util_template
* @file
* @brief Template Factory definition 
*  
* - Supports: All 
* - Created: 2005-08-06
* - Creator: Olivier Delannoy
* - $Revision $
* - Changelog: 
*    * 2005-08-07 : Integration into libutil 
*        - Moved from Util::TL to Util namespace
*    * 2005-05-31 : Initial version  
*******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

*******************************************************************************/
#ifndef UTIL_SINGLETON_HH
#define UTIL_SINGLETON_HH 1
#include "util_namespace.hh"
#include <cassert>
namespace Util {
    /**
     * @ingroup util_template
     * @brief Singleton DP Template Declaration
     *
     * This template implements the corresponding 
     * Design Pattern named Singleton. It's a way to 
     * constraints the application to be able to have only 
     * one object of all class which inherits from this 
     * template. 
     * 
     * @note The inheritance should be protected. 
     *
     * @code
     * #include "Singleton.hh"
     * #include <iostream>
     * // Protected Inheritance is a requierment
     * class Test : protected Util::Singleton<Test>
     * {
     * public:
     *  Test() 
     *    {
     *      std::cout <<"Creating an instance" << std::endl;
     *      
     *    }
     *  ~Test() 
     *    {
     *      std::cout << "Destroying an instance" << std::endl;
     *    }
     *  
     *  void print() 
     *    {
     *      std::cout << "Coucou" << std::endl;
     *    }
     *  
     *  using Util::Singleton<Test>::getSingleton;
     *  using Util::Singleton<Test>::getSingletonPtr;
     *
     * };
     *
     *
     * int main(void) 
     * {
     *   Test test;
     *   Test *ptr = Test::getSingletonPtr();
     *   assert(&test == ptr);
     *   ptr->print();
     *   Test::getSingleton().print();
     *  
     *   // Test test2; This will failed 
     *   return 0;
     * }
     *
     * @endcode
     */
    template <class T> 
    class Singleton 
    {
    public:
        /**
         * Get a reference to the unique instance of this class. If
         * the singleton as not yet been instantiated or as been
         * destroyed an assertion occured 
         * @return a reference to the instance 
         */
        static T& getSingleton(void) 
        {
            assert(mInstance); 
            return *mInstance;
        }
        /**
         * Get a pointer to the unique instance of this class. It
         * might be really usefull to new the unique instance once
         * for all 
         */
        static T* getSingletonPtr(void)
        {
            return mInstance;
        }
            
    protected:
        /**
         * Default constructor
         */
        Singleton() 
        {
            assert(mInstance == 0);
            mInstance = static_cast<T*>(this);
        }
            
        /**
         * Destructor
         */         
        virtual ~Singleton() 
        {
            mInstance = 0;
        }
            
                         
    private:
        /**
         * Store the static reference to the unique instance
         */
        static T *mInstance;
        /**
         * Disabled operation 
         */
        Singleton(const Singleton&) 
        {
            assert(0);
        }
        /**
         * This operator is disabled 
         * @return a reference to the current object 
         */
        Singleton& operator=(const Singleton&) 
        {
            assert(0);
            return *this;
        }
    };
    template <class T> T* Singleton<T>::mInstance = 0;
}   
#endif
/*******************************************************************************/      



