/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @ingroup util_system
 * @file
 * @brief Class Thread declaration
 *  
 * - Supports:
 *   - System 
 *       - UNIX/Pthread 
 *       - Win32 
 *   - SDL  
 * - Created: 2005-08-06
 * - Creator: Olivier Delannoy
 * - $Revision $
 * - Changelog: 
 *    
 *    * 2005-08-06 : Initial version  
 *******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_THREAD_HH
#define UTIL_THREAD_HH 1
#include "util_namespace.hh"
#include <iostream>
namespace Util {
    /**
     * @ingroup util_system
     * @brief Thread class 
     * 
     * A thread provide an alternative execution flot within the same
     * process.  the Thread class is abstract and you should implement
     * runImpl in all subclass. This class might rely on the system
     * wide thread API or on SDL. The end-user does not have to be
     * changed depending on the underlying thread api in use.
     * 
     * - Supported platforms:
     *   - System 
     *       - UNIX/Pthread (Linux)
     *       - Win32 (XP)
     *   - SDL (Linux) 
     */
    class UTIL_EXPORT Thread 
    {
    public:
        /**
         * Default Constructor 
         */
        Thread(void);
        /**
         * Destructor 
         */
        virtual ~Thread(void);

        /**
         * Start light process execution 
         * @note This function should only be called once per object.  
         */
        void start(void);
        
        /**
         * Terminate a light process execution 
         * @note this function should only be called once and after the thread was started 
         */
        void interrupt(void);
        
        /**
         * Test if the Thread has been started 
         * @return true if start() has been called previously 
         */
        bool isStarted(void) const;

        /**
         * Test if the Thread has terminate its execution 
         * 
         * @return true if the thread as been started and has leave the
         * runImpl method.
         * 
         */
        bool isFinished(void) const;

        /**
         * Test if the Thread is currently runing
         * @return true if the thread has been started but is not finished yet. 
         */
        bool isRuning(void) const;

        /**
         * wait for the thread to terminate 
         */ 
        void join(void);

        /**
         * Return the id of the thread 
         * @return the thread id
         */
        const String& identifier(void) const;

        /**
         * Format object to a text representation 
         * @param out The stream used for formatting 
         */ 
        virtual void print(std::ostream& out) const;
        
    protected:
        /**
         * This function define the body of the light process. It's quite 
         * similar to the function main of a program once you quit the 
         * function the process is finished 
         */
        virtual void run(void) = 0;
       
      
    private:
        /** 
         * @brief Thread Implementation class
         * @ingroup util_system
         * 
         * A Thread implementation provides a concret implementation of
         * the Thread class. It encompass all portability problem associated
         * to the use of different underlying library. 
         */
        class ThreadImpl 
        { 
        public: 
			virtual ~ThreadImpl() {}
            virtual void start(Thread*) = 0; 
            virtual void interrupt(void) = 0; 
            virtual void join(void) = 0; 
            virtual String identifier(void) = 0;
        };
        /**
         * @brief State of a Thread 
         * 
         * Enumerates the differents states of a Thread. 
         */
        enum ThreadState
        {
            THREAD_WAITING, /**< The thread has not yet been started */
            THREAD_RUNNING, /**<  The thread has been started */
            THREAD_FINISHED,/**< The thread is finished */
            THREAD_ERROR,  /**< The thread is in an error state */ 
        };

        /** Store the Thread identifier */
        String mIdentifier;

        /** Store the current state of the Thread */
        ThreadState mState;

        /** Store the concret implementation of the Thread lightweigth process */
        ThreadImpl* mImpl;
        /** Disabled Copy Constructor */
        Thread(Thread& src);
        /** Disabled Assignment operator */
        Thread& operator=(Thread& src);
    };
}
std::ostream& operator<<(std::ostream& out, const Util::Thread& obj);

#ifdef UTIL_OPT_INLINE 
#  include "Thread.icc"
#endif 
#endif
/*******************************************************************************/      



